import { Injectable, Inject } from '@angular/core';
import { Store, select } from '@ngrx/store';

import {
  Type,
} from '@loafer/core';

import { TypeUtil } from '@loafer/core/util/TypeUtil';

import {
  Class,
  Metadata,
} from '@loafer/core/reflect';

import { RPCSubscriberDecorator } from '../decorator';

import {
  RPCClientNotificationCodec,
} from '../protocol';

@Injectable()
export class RPCSubscribeService {
  private subscribers: Set<any>;

  constructor(
  ) {
    this.subscribers = new Set();
  }

  public addSubscriber(subscriber: Type<any>): void {
    const type = TypeUtil.getType(subscriber);

    if (this.subscribers.has(subscriber)) {
      console.log(`Subscriber[${type.name}] has been added`);
      return;
    }

    this.subscribers.add(subscriber);

    const clazz = Class.forType(type);
    if (undefined === clazz) {
      console.log(`Type[${subscriber.name}] is not decorated type`);
      return;
    }

    const methods = clazz.getMethods();
    methods.forEach((method, propertyKey) => {
      const annon = method.getAnnotation(RPCSubscriberDecorator);
      console.log(`subscriber method[${annon.attribute.method}]`);
    });
  }

  public notify(codec: RPCClientNotificationCodec): void {
    console.log(`notify method[${codec.method()}] params[${codec.params()}]`);
  }
}
